_antonyms = {
    "evne": "manglende evne",
    "abnorm": "normal",
    "afskaffe": "etablere",
    "fravær": "tilstedeværelse",
    "fraværende": "til stede",
    "fremtid": "forbi",
    "absolut": "i forhold",
    "afholde sig": "forbruge",
    "abstrakt": "beton",
    "overflod": "mangel",
    "rigelig": "knap",
    "acceptere": "afvise",
    "indrømme": "nægte",
    "acceptabel": "ufremkommelig",
    "accept": "afvisning",
    "tilgængelig": "utilgængelig",
    "nøjagtig": "forkert",
    "erkendte": "uerkendt",
    "bekendtskab": "fremmed",
    "handling": "passivitet",
    "aktiv": "rolige",
    "bevaret": "ødelagt",
    "støjende": "rolige",
    "aktivitet": "inaktivitet",
    "faktiske": "potentiel",
    "tilføje": "trække fra",
    "ud": "subtraktion",
    "tilstrækkelig": "utilstrækkelig",
    "adjektiv": "materielle",
    "realitetsbehandling": "afvises",
    "voksen": "ungdoms",
    "rykke": "trække sig tilbage",
    "foran": "bag-",
    "veer": "tilbage",
    "fordel": "ulempe",
    "bekræftelse": "vending",
    "bekræftende": "negativ",
    "neutral": "negativ",
    "agterste": "forgrunden",
    "alder": "forynge",
    "alderen": "ung",
    "agitation": "stilhed",
    "enig": "være uenig",
    "behagelig": "ubehagelig",
    "aftale": "uenighed",
    "ændret": "uændret",
    "skifte": "modsat",
    "amatør": "professionel",
    "tvetydig": "utvetydige",
    "venskabelig": "fjendtlig",
    "venlige": "uvenlig",
    "amfibie": "jordbaserede",
    "anal": "mundtlig",
    "analog": "homologe",
    "analyse": "syntese",
    "analytisk": "syntetisk",
    "forfader": "efterkommer",
    "kantet": "afrundet",
    "animere": "livløse",
    "svar": "spørgsmål",
    "antagonistiske": "synergistiske",
    "forudgående": "efterfølgende",
    "anterior": "posterior",
    "antiseptisk": "septisk",
    "udseende": "forsvinden",
    "påskønnelse": "afskrivninger",
    "godkendelse": "misbilligelse",
    "godkende": "misbillige",
    "akvatiske": "amfibie",
    "jordbaserede": "amfibie",
    "vandig": "magmatiske",
    "arm": "afvæbne",
    "bevæbnet": "ubevæbnet",
    "ankomme": "forlade",
    "bestige": "ned",
    "opstigning": "afstamning",
    "i søvn": "vågen",
    "samstemmende udtalelse": "uenighed",
    "asymmetri": "symmetri",
    "ateisme": "teisme",
    "vedhæfte": "løsrive",
    "angreb": "forsvare",
    "rejse tiltale": "forsvare",
    "opmærksomhed": "uopmærksomhed",
    "attraktion": "frastødning",
    "hørbar": "uhørlig",
    "augmentation": "formindskelse",
    "tante": "onkel",
    "autoriseret": "uberettiget",
    "automatisk": "brugervejledning",
    "undgå": "konfrontere",
    "vågen": "i søvn",
    "tilbage": "veer",
    "trække sig tilbage": "rykke",
    "bag-": "hoved",
    "baggrund": "forgrunden",
    "baglæns": "frem",
    "bagside": "forsiden",
    "dårlig": "godt",
    "ond": "dydige",
    "ondskab": "godhed",
    "afbalanceret": "ubalanceret",
    "bjørn": "tyr",
    "smuk": "grim",
    "skønhed": "grimhed",
    "begynde": "ende",
    "starten": "midten",
    "tidlig": "sent",
    "tro": "vantro",
    "bøje": "glatte",
    "godartet": "ondartet",
    "stor": "lille",
    "fødsel": "død",
    "biseksuel": "homoseksuel",
    "sort": "hvid",
    "velsigne": "forbande",
    "velsignet": "forbandet",
    "ublodig": "blodig",
    "blodig": "ublodig",
    "sløring": "fokus",
    "koge": "fryse",
    "dristig": "sky",
    "dristighed": "frygtsomhed",
    "låne": "låne",
    "låntager": "långiver",
    "bund": "top",
    "bundet": "ubundet",
    "dreng": "pige",
    "boykot": "nedladende",
    "pause": "reparation",
    "skør": "ubrydelig",
    "lysne": "mørkere",
    "gået i stykker": "ubrudt",
    "bror": "søster",
    "tyr": "bjørn",
    "travl": "ledig",
    "arbejde": "ledig",
    "købe": "sælge",
    "berolige": "bekymre",
    "stilhed": "agitation",
    "i stand til at": "stand",
    "kapacitet": "uarbejdsdygtighed",
    "kontanter": "kredit",
    "betalingskort": "kredit",
    "centrum": "venstre",
    "cerebral": "følelsesmæssig",
    "sikkerhed": "usikkerhed",
    "lave om": "bliv",
    "afgår": "bliv",
    "bevæge sig": "bliv",
    "foranderlig": "uforanderlige",
    "oplade": "udledning",
    "hverve": "udledning",
    "billig": "dyrt",
    "opmuntre": "brokke sig",
    "glad": "sorgfuld",
    "barn": "forælder",
    "civile": "militær",
    "påstand": "miste",
    "klarhed": "gennemsigtighed",
    "fremtrædende": "ubemærkethed",
    "gennemsigtighed": "gennemsigtighed",
    "ren": "urene",
    "gejstlighed": "lægfolk",
    "præst": "lægmand",
    "tæt": "åben",
    "lukket": "åben",
    "lukke": "åben",
    "nærhed": "åbenhed",
    "kystnære": "indre",
    "sammenhængende": "diskontinuerlig",
    "kold": "hed",
    "sikkerhedsstillelse": "lineal",
    "komedie": "tragedie",
    "komfort": "ubehag",
    "almindelige": "ualmindelig",
    "kompatibel": "trodsig",
    "kompetence": "inkompetence",
    "kompetent": "inkompetent",
    "konkurrence": "samarbejde",
    "brokke sig": "opmuntre",
    "fuldstændighed": "ufuldstændige",
    "kompleks": "enkel",
    "forbindelse": "enkel",
    "komplicere": "forenkle",
    "konkave": "konveks",
    "indbildskhed": "ydmyghed",
    "stolthed": "ydmyghed",
    "koncentration": "fordeling",
    "koncentriske": "excentrisk",
    "forestilling": "misforståelse",
    "berørte": "ubekymret",
    "beton": "abstrakt",
    "bekræftet": "ubekræftet",
    "konfrontere": "undgå",
    "konjunktiv": "disjunktiv",
    "forbinde": "koble fra",
    "forbundet": "usammenhængende",
    "bevidsthed": "bevidstløshed",
    "konservativ": "liberal",
    "spare": "spild",
    "hensynsfuld": "ubetænksom",
    "konsistens": "uoverensstemmelse",
    "konsonant": "vokal",
    "iøjnefaldende": "upåfaldende",
    "forfatningsmæssig": "forfatningsstridig",
    "forbruge": "afholde sig",
    "indhold": "utilfredshed",
    "løbende": "sporadisk",
    "fortsættelse": "ophør",
    "blive ved": "afbryde",
    "kontinuitet": "diskontinuitet",
    "kontrakt": "udvide",
    "kontraheret": "udvidet",
    "sammentrækning": "udvidelse",
    "praktisk": "ubelejligt",
    "konventionel": "ukonventionelle",
    "konvergerer": "divergere",
    "konvergens": "divergens",
    "konvergent": "divergerende",
    "konveks": "konkave",
    "overbevist": "ikke overbevist",
    "fedt nok": "varme",
    "samarbejde": "konkurrence",
    "kosmopolitisk": "provins-",
    "mod": "fejhed",
    "dække over": "afdække",
    "skjult": "åbenlys",
    "fejhed": "mod",
    "kredit": "betalingskort",
    "kreditor": "debitor",
    "kritisere": "ros",
    "skrig": "grine",
    "helbredes": "uhelbredelig",
    "forbande": "velsigne",
    "forbandet": "velsignet",
    "buet": "lige",
    "skære": "uncut",
    "fare": "sikkerhed",
    "farligt": "sikker",
    "mørk": "lys",
    "mørkere": "lysne",
    "mørke": "lethed",
    "tyngde": "lethed",
    "datter": "søn",
    "dag": "nat",
    "døv": "høring",
    "død": "fødsel",
    "debitor": "kreditor",
    "anstændighed": "uanstændighed",
    "anstændig": "uanstændig",
    "løvfældende": "stedsegrønne",
    "afgørelse": "ubeslutsomhed",
    "afgørende": "ubeslutsom",
    "nedgang": "forbedring",
    "formindske": "øge",
    "er faldet": "øget",
    "formindskelsen": "tilvækst",
    "nederlag": "sejr",
    "forsvare": "rejse tiltale",
    "sagsøgt": "sagsøger",
    "forsvar": "retsforfølgelse",
    "defensiv": "offensiv",
    "trodsig": "kompatibel",
    "underskud": "at føre",
    "defineret": "undefined",
    "deflation": "inflation",
    "forsinke": "siv",
    "drysse": "siv",
    "efterspørgsel": "levere",
    "nægte": "acceptere",
    "afvise": "acceptere",
    "afhængig": "uafhængig",
    "depositum": "trække",
    "afskrivninger": "påskønnelse",
    "deprimerende": "glad",
    "ned": "bestige",
    "efterkommer": "forfader",
    "afstamning": "opstigning",
    "beskrivende": "præskriptive",
    "ønskelig": "uønsket",
    "fortvivlelse": "håber",
    "desperat": "håber",
    "desperation": "håber",
    "ødelagt": "bevaret",
    "løsrive": "vedhæfte",
    "decentralisering": "udvikling",
    "afvige": "lige",
    "ulige": "lige",
    "forskel": "ensartethed",
    "svært": "let",
    "urolig": "let",
    "vanskelighed": "lethed",
    "flittig": "uagtsom",
    "fortynding": "koncentration",
    "fordeling": "koncentration",
    "formindskelse": "augmentation",
    "snavset": "ren",
    "urene": "ren",
    "deaktivere": "gøre det muligt for",
    "ulempe": "fordel",
    "være uenig": "enig",
    "ubehagelig": "behagelig",
    "uenighed": "samstemmende udtalelse",
    "forsvinden": "udseende",
    "misbilligelse": "godkendelse",
    "misbillige": "godkende",
    "afvæbne": "arm",
    "udledning": "hverve",
    "fralægger": "påstand",
    "miste": "påstand",
    "ubehag": "komfort",
    "koble fra": "forbinde",
    "utilfredshed": "tilfredshed",
    "ophør": "fortsættelse",
    "afbryde": "blive ved",
    "diskontinuitet": "kontinuitet",
    "diskontinuerlig": "sammenhængende",
    "afskrække": "tilskynde",
    "modløshed": "opmuntring",
    "frigøre": "engagere",
    "uærlig": "ærlig",
    "uærlighed": "ærlighed",
    "vanære": "ære",
    "desinficere": "inficere",
    "smuldre": "integrere",
    "disjunktiv": "konjunktiv",
    "kan ikke lide": "smag",
    "løsne": "binde",
    "ulydighed": "lydighed",
    "ulydig": "lydig",
    "være ulydig": "adlyde",
    "sygdom": "bestille",
    "uordentligt": "velordnet",
    "uorganiseret": "organiseret",
    "uorganiserede": "organiseret",
    "utilfreds": "tilfreds",
    "modbevise": "bevise",
    "diskvalificere": "kvalificere",
    "berygtet": "velrenommeret",
    "miskredit": "omdømme",
    "respektløshed": "respekt",
    "respektløst": "respektfuld",
    "bjørnetjeneste": "service",
    "ulig": "lignende",
    "dissonans": "harmoni",
    "afholde": "overtale",
    "distal": "proximal",
    "fjern": "tæt",
    "åben": "lukke",
    "skelnes": "skelnes",
    "mistillid": "tillid",
    "døgnets": "natlig",
    "divergere": "konvergerer",
    "divergens": "konvergens",
    "divergerende": "konvergent",
    "dele": "forene",
    "enkeltvis": "formere sig",
    "delt op": "forenet",
    "delelig": "udelelig",
    "indenlandske": "udenlandsk",
    "hjemmehørende": "udenlandsk",
    "dominerende": "underordnet",
    "hvilende": "aktiv",
    "uddød": "bevaret",
    "inaktiv": "aktiv",
    "passiv": "aktiv",
    "rolige": "støjende",
    "dobbelt": "enkelt",
    "mange": "enkelt",
    "due": "høg",
    "dramatisk": "lyrik",
    "tør": "våd",
    "sur": "forsøde",
    "på grund": "unødig",
    "kedelig": "skærpe",
    "flade": "skærpe",
    "døende": "spirende",
    "jordisk": "himmelske",
    "lethed": "tyngde",
    "let": "urolig",
    "ebbe": "tidevand",
    "excentrisk": "koncentriske",
    "uddannet": "utrænede",
    "effektiv": "ineffektiv",
    "effektivitet": "ineffektivitet",
    "egress": "indtrængen",
    "elastisk": "uelastisk",
    "berettiget": "støtteberettigede",
    "følelsesmæssig": "cerebral",
    "empirisk": "teoretisk",
    "tomhed": "fylde",
    "tom": "fuld",
    "tynd": "tykkere",
    "gøre det muligt for": "deaktivere",
    "tilskynde": "afskrække",
    "opmuntring": "modløshed",
    "ende": "starten",
    "slutning": "starten",
    "midten": "tidlig",
    "endemisk": "kosmopolitisk",
    "provins-": "kosmopolitisk",
    "energisk": "sløv",
    "håndhæve": "fritaget",
    "engagere": "frigøre",
    "god fornøjelse": "lide",
    "gå ind": "afslut",
    "lige": "buet",
    "lighed": "ulighed",
    "ækvatoriale": "polær",
    "retfærdig": "urimelig",
    "slette": "optage",
    "etablere": "afskaffe",
    "begivenhedsrig": "begivenhedsløs",
    "stedsegrønne": "løvfældende",
    "udvikling": "decentralisering",
    "undtagelse": "inklusion",
    "fritaget": "håndhæve",
    "ånde ud": "inhalere",
    "eksisterende": "ikke-eksisterende",
    "afslut": "start",
    "udvide": "smal",
    "udvidet": "kontraheret",
    "udvidelse": "fleksion",
    "forventes": "uventet",
    "dyrt": "billig",
    "oplevet": "uerfarne",
    "eksport": "importere",
    "udtrykke": "lokal",
    "generel": "bestemt",
    "national": "international",
    "ydre": "interiør",
    "ekstern": "indre",
    "eksternt": "internt",
    "slukke": "antænde",
    "extramural": "intramuralt",
    "ekstraordinær": "almindelig",
    "extrinsic": "iboende",
    "svigtende": "passerer",
    "løb": "stående",
    "fiasko": "succes",
    "trofast": "utro",
    "efterår": "stige",
    "sæt": "stige",
    "fejlbarlig": "ufejlbarlig",
    "faldende": "stigende",
    "falskhed": "sandhed",
    "falskheden": "sandhed",
    "berømmelse": "skændsel",
    "velkendt": "ukendt",
    "fancy": "almindeligt",
    "fed": "tynd",
    "fuld": "tom",
    "tyk": "tynd",
    "tykkere": "tynd",
    "far": "mor",
    "fauna": "flora",
    "gunstig": "ugunstige",
    "føderale": "ét stykke",
    "kvinde": "han-",
    "feminin": "kastrat",
    "kvindelighed": "maskulinitet",
    "frodige": "steril",
    "troskab": "utroskab",
    "figur": "jord",
    "fylde": "tomhed",
    "find": "tabe",
    "hold op": "start",
    "begrænset": "ubegrænset",
    "brand": "leje",
    "fleksion": "udvidelse",
    "flyde": "håndvask",
    "kilde": "håndvask",
    "flora": "fauna",
    "fokus": "sløring",
    "fjende": "ven",
    "folde": "udfolde",
    "følge efter": "gå forud",
    "lykkes": "gå forud",
    "tilhænger": "leder",
    "dårskab": "visdom",
    "tåbelig": "klog",
    "fod": "hoved",
    "hale": "hoved",
    "forbyde": "tilladelse",
    "forgrunden": "baggrund",
    "udenlandsk": "hjemmehørende",
    "glemme": "husk",
    "tilgivende": "nådesløs",
    "formalisme": "efterligning",
    "heldig": "uheldig",
    "frem": "bagside",
    "forsiden": "bagside",
    "modbydeligt": "retfærdig",
    "urimelig": "retfærdig",
    "fundet": "faret vild",
    "broderlige": "identisk",
    "fryse": "koge",
    "hyppig": "sjælden",
    "hyppigt": "sjældent",
    "ven": "fjende",
    "hoved": "hale",
    "funktionel": "organisk",
    "uorganisk": "organisk",
    "gevinst": "tab",
    "gasformig": "solid",
    "hule": "solid",
    "samle": "sprede",
    "generalitet": "særegenhed",
    "gavmild": "nærig",
    "pige": "dreng",
    "godt": "ond",
    "godhed": "ondskab",
    "gradvis": "pludselig",
    "taknemmelig": "utaknemmelig",
    "større": "mindre",
    "grøn": "moden",
    "brutto": "net",
    "jord": "figur",
    "skyld": "uskyld",
    "skyldig": "uskyldig",
    "lykkeligt": "desværre",
    "lykke": "ulykkelighed",
    "lykkelig": "ulykkelig",
    "hårdhed": "blødhed",
    "loudness": "blødhed",
    "skarphed": "blødhed",
    "harmoni": "dissonans",
    "had": "elsker",
    "hadefulde": "elskelig",
    "høg": "due",
    "sund og rask": "usund",
    "høring": "døv",
    "varme": "fedt nok",
    "himmelske": "jordisk",
    "heterogenitet": "homogenitet",
    "skjule": "at vise",
    "leje": "brand",
    "hit": "gå glip af",
    "hellig": "uhellig",
    "homogenitet": "heterogenitet",
    "homologe": "analog",
    "homoseksuel": "biseksuel",
    "ærlig": "uærlig",
    "ærlighed": "uærlighed",
    "ære": "vanære",
    "håber": "desperation",
    "håbefuld": "håbløs",
    "håbløs": "håbefuld",
    "vandret": "lodret",
    "fjendtlig": "venlige",
    "uvenlig": "venlig",
    "hed": "kold",
    "ydmyg": "stolt",
    "ydmyghed": "stolthed",
    "sulten": "tørstig",
    "identisk": "broderlige",
    "ledig": "arbejde",
    "magmatiske": "vandig",
    "antænde": "slukke",
    "ignorere": "varsel",
    "ulovlig": "lovlig",
    "ulovlighed": "lovligheden",
    "ulæselig": "læselig",
    "ulogisk": "logisk",
    "efterligning": "formalisme",
    "uvæsentlig": "vigtig",
    "umodenhed": "modenhed",
    "umålelig": "målbar",
    "umoralsk": "moralsk",
    "umoral": "moral",
    "udødelig": "dødelig",
    "udødelighed": "dødelighed",
    "uforanderlige": "foranderlig",
    "upartiskhed": "partiskhed",
    "ufremkommelig": "acceptabel",
    "utålmodighed": "tålmodighed",
    "utålmodig": "patient",
    "umærkelig": "mærkbar",
    "ufuldkommenhed": "fuldkommenhed",
    "uigennemtrængelig": "gennemtrængelig",
    "utilladeligt": "tilladelig",
    "upersonlig": "personlig",
    "uhøfligt": "høflig",
    "importere": "eksport",
    "vigtig": "uvæsentlig",
    "umulighed": "mulighed",
    "impotent": "potent",
    "usandsynlig": "sandsynlig",
    "forbedret": "uforbedrede",
    "forbedring": "nedgang",
    "uforsigtig": "forsigtig",
    "urenhed": "renhed",
    "manglende evne": "evne",
    "utilgængelig": "tilgængelig",
    "forkert": "nøjagtig",
    "passivitet": "handling",
    "inaktivitet": "aktivitet",
    "utilstrækkelig": "tilstrækkelig",
    "afvises": "realitetsbehandling",
    "livløse": "animere",
    "uopmærksomhed": "opmærksomhed",
    "uhørlig": "hørbar",
    "stand": "i stand til at",
    "uarbejdsdygtighed": "kapacitet",
    "hældende": "vandret",
    "lodret": "vandret",
    "inklusion": "undtagelse",
    "usammenhængende": "forbundet",
    "uforenelig": "kompatibel",
    "inkompetence": "kompetence",
    "inkompetent": "kompetent",
    "ufuldstændige": "fuldstændighed",
    "ubetænksom": "hensynsfuld",
    "uoverensstemmelse": "konsistens",
    "upåfaldende": "iøjnefaldende",
    "ubelejligt": "praktisk",
    "øge": "formindske",
    "øget": "er faldet",
    "tilvækst": "formindskelsen",
    "uhelbredelig": "helbredes",
    "uanstændighed": "anstændighed",
    "uanstændig": "anstændig",
    "ubeslutsomhed": "afgørelse",
    "ubeslutsom": "afgørende",
    "uafhængig": "afhængig",
    "udelelig": "delelig",
    "indendørs": "udendørs",
    "ineffektiv": "effektiv",
    "ineffektivitet": "effektivitet",
    "uelastisk": "elastisk",
    "støtteberettigede": "berettiget",
    "ulighed": "lighed",
    "uretfærdig": "retfærdig",
    "uerfarne": "oplevet",
    "ufejlbarlig": "fejlbarlig",
    "skændsel": "berømmelse",
    "inficere": "desinficere",
    "underlegen": "overlegen",
    "underlegenhed": "overlegenhed",
    "utroskab": "troskab",
    "uendelig": "begrænset",
    "inflation": "deflation",
    "informeret": "uvidende",
    "sjælden": "hyppig",
    "sjældent": "hyppigt",
    "indtrængen": "egress",
    "beboet": "ubeboet",
    "inhalere": "ånde ud",
    "uretfærdighed": "retfærdighed",
    "indre": "ekstern",
    "uskyld": "skyld",
    "uskyldig": "skyldig",
    "harmløs": "skadelige",
    "inoperativ": "operative",
    "sindssyg": "sane",
    "sindssyge": "fornuft",
    "usikkerhed": "sikkerhed",
    "ubetydelighed": "betydning",
    "ubetydelig": "væsentlig",
    "uopløselig": "opløseligt",
    "insolvens": "solvens",
    "konkurs": "opløsningsmiddel",
    "ustabilitet": "stabilitet",
    "medvirkende": "vokal",
    "insufficiens": "selvforsyning",
    "utilstrækkeligt": "tilstrækkeligt",
    "integrere": "smuldre",
    "integration": "adskillelse",
    "intelligens": "dumhed",
    "forståelig": "uforståelig",
    "beregnet": "utilsigtet",
    "med vilje": "utilsigtet",
    "interesseret": "uinteresseret",
    "interessant": "uinteressant",
    "interiør": "ydre",
    "mellemprodukt": "terminal",
    "internt": "eksternt",
    "international": "national",
    "lokal": "generel",
    "intolerance": "tolerance",
    "intolerant": "tolerant",
    "umedgørlig": "medgørlig",
    "intramuralt": "extramural",
    "iboende": "extrinsic",
    "usynlighed": "sigtbarhed",
    "usynlig": "synlig",
    "ufrivillig": "frivillig",
    "irrationel": "rationel",
    "uregelmæssig": "fast",
    "uregelmæssighed": "regelmæssighed",
    "irrelevant": "relevant",
    "uansvarlig": "ansvarlig",
    "respektløs": "ærbødig",
    "problem": "minde om",
    "samling": "adskille",
    "glæde": "sorg",
    "junior-": "senior-",
    "retfærdighed": "uretfærdighed",
    "ungdoms": "voksen",
    "kern": "kern",
    "venlig": "uvenlig",
    "konge": "dronning",
    "lægfolk": "gejstlighed",
    "sent": "tidlig",
    "grine": "skrig",
    "lovlig": "ulovlig",
    "slap": "anspændt",
    "afslappet": "anspændt",
    "lægmand": "præst",
    "at føre": "underskud",
    "leder": "tilhænger",
    "forlade": "ankomme",
    "venstre": "centrum",
    "gyldige": "ulovlig",
    "lovligheden": "ulovlighed",
    "legalisere": "fredløs",
    "læselig": "ulæselig",
    "långiver": "låntager",
    "forlænge": "forkorte",
    "mindre": "større",
    "sløv": "energisk",
    "niveau": "hæve",
    "nederste": "hæve",
    "liberal": "konservativ",
    "ligge": "stå",
    "udbytte": "stå",
    "lys": "mørk",
    "smag": "kan ikke lide",
    "lineal": "sikkerhedsstillelse",
    "væske": "gasformig",
    "solid": "hule",
    "bestemt": "generel",
    "beliggenhed": "studie",
    "låse": "låse op",
    "lodge": "løsne",
    "logisk": "ulogisk",
    "lang": "kort",
    "høj": "kort",
    "tabe": "find",
    "tab": "gevinst",
    "faret vild": "fundet",
    "elskelig": "hadefulde",
    "elsker": "had",
    "lyrik": "dramatisk",
    "flertal": "mindretal",
    "lave": "pause",
    "reparation": "pause",
    "han-": "kvinde",
    "malign": "godartet",
    "ondartet": "godartet",
    "brugervejledning": "automatisk",
    "markeret": "umærket",
    "gift": "ugift",
    "maskulin": "feminin",
    "kastrat": "feminin",
    "maskulinitet": "kvindelighed",
    "maske": "demaskere",
    "materiale": "uvæsentlig",
    "mødres": "faderlig",
    "modenhed": "umodenhed",
    "maksimum": "minimum",
    "målbar": "umålelig",
    "mental": "fysisk",
    "barmhjertig": "nådesløs",
    "nådesløs": "tilgivende",
    "militær": "civile",
    "sind": "glemme",
    "husk": "glemme",
    "minimum": "maksimum",
    "mindretal": "flertal",
    "misforståelse": "forestilling",
    "gå glip af": "hit",
    "kindtand": "molekylær",
    "molekylær": "kindtand",
    "moralsk": "umoralsk",
    "moral": "umoral",
    "dødelig": "udødelig",
    "dødelighed": "udødelighed",
    "mor": "far",
    "formere sig": "enkeltvis",
    "forene": "dele",
    "lavpunkt": "zenit",
    "smal": "udvide",
    "spirende": "døende",
    "negativ": "neutral",
    "uagtsom": "flittig",
    "nevø": "niece",
    "net": "brutto",
    "niece": "nevø",
    "nat": "dag",
    "natlig": "døgnets",
    "nominel": "ægte",
    "uvirkelig": "ægte",
    "ikke-eksisterende": "eksisterende",
    "normal": "abnorm",
    "varsel": "ignorere",
    "skadelige": "harmløs",
    "numerisk": "verbal-",
    "lydighed": "ulydighed",
    "lydig": "ulydig",
    "adlyde": "være ulydig",
    "objektiv": "subjektiv",
    "obligatorisk": "valgfri",
    "skrå": "vinkelret",
    "ubemærkethed": "fremtrædende",
    "gennemsigtighed": "gennemsigtighed",
    "optaget": "ubesatte",
    "forbrydelse": "forsvar",
    "retsforfølgelse": "forsvar",
    "offensiv": "defensiv",
    "officiel": "uofficiel",
    "åbenhed": "nærhed",
    "operative": "inoperativ",
    "imod": "enstemmigt",
    "modsat": "skifte",
    "optimisme": "pessimisme",
    "optimist": "pessimist",
    "valgfri": "obligatorisk",
    "mundtlig": "anal",
    "bestille": "sygdom",
    "velordnet": "uordentligt",
    "almindelig": "ekstraordinær",
    "organisk": "uorganisk",
    "organiseret": "uorganiserede",
    "ortodokse": "uortodokse",
    "udendørs": "indendørs",
    "fredløs": "legalisere",
    "overvurdere": "undervurdere",
    "overliggende": "overflade",
    "åbenlys": "skjult",
    "pakke": "pakker ud",
    "smertefuld": "smertefri",
    "smertefri": "smertefuld",
    "parallel": "skrå",
    "vinkelret": "skrå",
    "forælder": "barn",
    "partiskhed": "upartiskhed",
    "særegenhed": "generalitet",
    "til dels": "helt",
    "passerer": "løb",
    "stående": "løb",
    "forbi": "fremtid",
    "til stede": "fraværende",
    "faderlig": "mødres",
    "tålmodighed": "utålmodighed",
    "patient": "utålmodig",
    "nedladende": "boykot",
    "fred": "krig",
    "straf": "belønning",
    "mærkbar": "umærkelig",
    "fuldkommenhed": "ufuldkommenhed",
    "gennemtrængelig": "uigennemtrængelig",
    "tilladelig": "utilladeligt",
    "eftergivende": "forebyggende",
    "tilladelse": "forbyde",
    "personlig": "upersonlig",
    "overtale": "afholde",
    "pessimisme": "optimisme",
    "pessimist": "optimist",
    "fysisk": "mental",
    "almindeligt": "fancy",
    "sagsøger": "sagsøgt",
    "tilfreds": "utilfreds",
    "pluralis": "ental",
    "polær": "ækvatoriale",
    "høflig": "uhøfligt",
    "populær": "upopulær",
    "porøsitet": "soliditet",
    "mulighed": "umulighed",
    "posterior": "anterior",
    "potent": "impotent",
    "potentiel": "faktiske",
    "fattigdom": "rigdom",
    "magtfulde": "magtesløs",
    "magtesløs": "magtfulde",
    "ros": "kritisere",
    "gå forud": "lykkes",
    "foregående": "efterfølgende",
    "præfiks": "endelse",
    "forberedelse": "løsning",
    "forberedt": "uforberedt",
    "præskriptive": "beskrivende",
    "tilstedeværelse": "fravær",
    "forebyggende": "eftergivende",
    "primær": "sekundær",
    "privat": "offentligt",
    "sandsynlig": "usandsynlig",
    "produktiv": "uproduktive",
    "blasfemisk": "hellige",
    "professionel": "amatør",
    "fremskridt": "relatere",
    "progressiv": "regressiv",
    "prospektive": "tilbagevirkende kraft",
    "beskyttet": "ubeskyttede",
    "stolt": "ydmyg",
    "bevise": "modbevise",
    "proximal": "distal",
    "nærtliggende": "ultimative",
    "forsigtig": "uforsigtig",
    "offentlig": "privat",
    "offentligt": "privat",
    "offentliggjort": "upubliceret",
    "trække": "depositum",
    "punktlighed": "langsommelighed",
    "purge": "rehabilitere",
    "renhed": "urenhed",
    "skubbe": "trække",
    "kvalificere": "diskvalificere",
    "dronning": "konge",
    "spørgsmål": "svar",
    "tvivlsom": "ubestridelige",
    "hurtigt": "langsomt",
    "hæve": "nederste",
    "rationel": "irrationel",
    "ægte": "uvirkelig",
    "realistisk": "urealistisk",
    "minde om": "problem",
    "optage": "slette",
    "afstå": "handling",
    "relatere": "fremskridt",
    "regressiv": "progressiv",
    "fast": "uregelmæssig",
    "regelmæssighed": "uregelmæssighed",
    "reguleret": "ureguleret",
    "rehabilitere": "purge",
    "afvisning": "accept",
    "forynge": "alder",
    "relaterede": "ingen forbindelse",
    "i forhold": "absolut",
    "relevant": "irrelevant",
    "pålidelig": "upålidelige",
    "religiøs": "sekulære",
    "frastødning": "attraktion",
    "velrenommeret": "berygtet",
    "omdømme": "miskredit",
    "modstå": "overgivelse",
    "løsning": "forberedelse",
    "respekt": "respektløshed",
    "respektfuld": "respektløst",
    "ansvarlig": "uansvarlig",
    "behersket": "uhæmmet",
    "tilbagevirkende kraft": "prospektive",
    "ærbødig": "respektløs",
    "vending": "bekræftelse",
    "belønning": "straf",
    "ride": "gå",
    "moden": "grøn",
    "stige": "sæt",
    "stigende": "faldende",
    "rotor": "stator",
    "ruhed": "glathed",
    "rund": "firkant",
    "afrundet": "kantet",
    "landdistrikterne": "by-",
    "siv": "drysse",
    "hellige": "blasfemisk",
    "desværre": "lykkeligt",
    "sorg": "glæde",
    "ulykkelighed": "lykke",
    "sikker": "farligt",
    "ensartethed": "forskel",
    "sane": "sindssyg",
    "fornuft": "sindssyge",
    "tilfredshed": "utilfredshed",
    "krydrede": "usmagelig",
    "knap": "rigelig",
    "mangel": "overflod",
    "videnskabelig": "uvidenskabelig",
    "samvittighedsfulde": "skrupelløs",
    "sekundær": "primær",
    "sekulære": "religiøs",
    "adskillelse": "union",
    "selvisk": "uselvisk",
    "sælge": "købe",
    "senior-": "junior-",
    "adskille": "samling",
    "septisk": "antiseptisk",
    "service": "bjørnetjeneste",
    "slog sig ned": "uafviklede",
    "skærpe": "flade",
    "kort": "høj",
    "forkorte": "forlænge",
    "råbe": "hviske",
    "at vise": "skjule",
    "krympe": "strække",
    "side": "bund",
    "top": "bund",
    "betydning": "ubetydelighed",
    "væsentlig": "ubetydelig",
    "lignende": "ulig",
    "enkel": "forbindelse",
    "forenkle": "komplicere",
    "enkelt": "mange",
    "ental": "pluralis",
    "håndvask": "kilde",
    "søster": "bror",
    "søvn": "vågne",
    "langsomt": "hurtigt",
    "lille": "stor",
    "glathed": "ruhed",
    "blødhed": "skarphed",
    "soliditet": "porøsitet",
    "opløseligt": "uopløselig",
    "solvens": "insolvens",
    "opløsningsmiddel": "konkurs",
    "søn": "datter",
    "sorgfuld": "glad",
    "talt": "skriftlig",
    "uskrevet": "skriftlig",
    "sporadisk": "løbende",
    "sprede": "samle",
    "firkant": "rund",
    "stabilitet": "ustabilitet",
    "stabil": "ustabil",
    "stå": "udbytte",
    "start": "hold op",
    "stator": "rotor",
    "bliv": "bevæge sig",
    "steady": "ustadig",
    "stejl": "gradvis",
    "pludselig": "gradvis",
    "steril": "frodige",
    "stivne": "løsne",
    "nærig": "gavmild",
    "stormfuld": "berolige",
    "glatte": "bøje",
    "fremmed": "bekendtskab",
    "styrke": "svaghed",
    "strække": "krympe",
    "stærkt": "svagt",
    "studie": "beliggenhed",
    "dumhed": "intelligens",
    "subjektiv": "objektiv",
    "underordnet": "dominerende",
    "efterfølgende": "foregående",
    "materielle": "adjektiv",
    "trække fra": "tilføje",
    "subtraktion": "ud",
    "succes": "fiasko",
    "vellykket": "mislykket",
    "bukke under": "overleve",
    "lide": "god fornøjelse",
    "selvforsyning": "insufficiens",
    "tilstrækkeligt": "utilstrækkeligt",
    "endelse": "præfiks",
    "solopgang": "solnedgang",
    "solnedgang": "solopgang",
    "overlegen": "underlegen",
    "overlegenhed": "underlegenhed",
    "levere": "efterspørgsel",
    "understøttet": "ikke-understøttet",
    "overflade": "overliggende",
    "overgivelse": "modstå",
    "overleve": "bukke under",
    "sød": "sur",
    "våd": "tør",
    "forsøde": "sur",
    "symmetri": "asymmetri",
    "sympatisk": "usympatisk",
    "synergistiske": "antagonistiske",
    "syntese": "analyse",
    "syntetisk": "analytisk",
    "langsommelighed": "punktlighed",
    "smagfuld": "smagløs",
    "velsmagende": "smagløs",
    "smagløs": "velsmagende",
    "bud": "sej",
    "anspændt": "afslappet",
    "terminal": "mellemprodukt",
    "teisme": "ateisme",
    "teoretisk": "empirisk",
    "tørstig": "sulten",
    "tankevækkende": "tankeløs",
    "tankeløs": "tankevækkende",
    "sparsommelige": "ødsel",
    "tidevand": "ebbe",
    "binde": "løsne",
    "sky": "dristig",
    "frygtsomhed": "dristighed",
    "tolerance": "intolerance",
    "tolerant": "intolerant",
    "tandede": "tandløs",
    "tandløs": "tandede",
    "rørt": "uberørt",
    "sej": "bud",
    "medgørlig": "umedgørlig",
    "tragedie": "komedie",
    "tillid": "mistillid",
    "sandhed": "falskheden",
    "grimhed": "skønhed",
    "grim": "smuk",
    "ultimative": "nærtliggende",
    "uacceptabel": "acceptabel",
    "uerkendt": "erkendte",
    "uændret": "ændret",
    "entydig": "tvetydig",
    "ubevæbnet": "bevæbnet",
    "uberettiget": "autoriseret",
    "ubalanceret": "afbalanceret",
    "vantro": "tro",
    "ubundet": "bundet",
    "ubrydelig": "skør",
    "ubrudt": "gået i stykker",
    "uforanderlig": "foranderlig",
    "onkel": "tante",
    "ualmindelig": "almindelige",
    "ubekymret": "berørte",
    "ubekræftet": "bekræftet",
    "bevidstløshed": "bevidsthed",
    "forfatningsstridig": "forfatningsmæssig",
    "ukonventionelle": "konventionel",
    "ikke overbevist": "overbevist",
    "afdække": "dække over",
    "uncut": "skære",
    "undefined": "defineret",
    "undervurdere": "overvurdere",
    "uønsket": "ønskelig",
    "unødig": "på grund",
    "uuddannet": "uddannet",
    "utvetydige": "tvetydig",
    "begivenhedsløs": "begivenhedsrig",
    "uventet": "forventes",
    "utro": "trofast",
    "ukendt": "velkendt",
    "ugunstige": "gunstig",
    "udfolde": "folde",
    "uheldig": "heldig",
    "utaknemmelig": "taknemmelig",
    "ulykkelig": "lykkelig",
    "usund": "sund og rask",
    "uhellig": "hellig",
    "uforbedrede": "forbedret",
    "uvidende": "informeret",
    "ubeboet": "beboet",
    "uforståelig": "forståelig",
    "utilsigtet": "med vilje",
    "uinteresseret": "interesseret",
    "uinteressant": "interessant",
    "union": "adskillelse",
    "ét stykke": "føderale",
    "forenet": "delt op",
    "ubegrænset": "begrænset",
    "låse op": "låse",
    "umærket": "markeret",
    "ugift": "gift",
    "demaskere": "maske",
    "ubesatte": "optaget",
    "uofficiel": "officiel",
    "enstemmigt": "imod",
    "uortodokse": "ortodokse",
    "pakker ud": "pakke",
    "upopulær": "populær",
    "uforberedt": "forberedt",
    "uproduktive": "produktiv",
    "ubeskyttede": "beskyttet",
    "upubliceret": "offentliggjort",
    "ubestridelige": "tvivlsom",
    "urealistisk": "realistisk",
    "ureguleret": "reguleret",
    "ingen forbindelse": "relaterede",
    "upålidelige": "pålidelig",
    "uhæmmet": "behersket",
    "usmagelig": "krydrede",
    "uvidenskabelig": "videnskabelig",
    "skrupelløs": "samvittighedsfulde",
    "uselvisk": "selvisk",
    "uafviklede": "slog sig ned",
    "ustabil": "stabil",
    "ustadig": "steady",
    "mislykket": "vellykket",
    "ikke-understøttet": "understøttet",
    "usympatisk": "sympatisk",
    "uberørt": "rørt",
    "utrænede": "uddannet",
    "usædvanlig": "sædvanlig",
    "afsløre": "slør",
    "uforsigtige": "vagt",
    "uvilligt": "villigt",
    "uvilje": "villighed",
    "slappe af": "vind",
    "uværdig": "værdig",
    "by-": "landdistrikterne",
    "sædvanlig": "usædvanlig",
    "værdifuld": "værdiløs",
    "slør": "afsløre",
    "verbal-": "numerisk",
    "sejr": "nederlag",
    "dydige": "ond",
    "sigtbarhed": "usynlighed",
    "synlig": "usynlig",
    "vokal": "konsonant",
    "frivillig": "ufrivillig",
    "vågne": "søvn",
    "gå": "ride",
    "aftagende": "voks",
    "krig": "fred",
    "vagt": "uforsigtige",
    "spild": "spare",
    "ødsel": "sparsommelige",
    "voks": "aftagende",
    "svagt": "stærkt",
    "svaghed": "styrke",
    "rigdom": "fattigdom",
    "vægtløs": "vægtig",
    "vægtig": "vægtløs",
    "hviske": "råbe",
    "hvid": "sort",
    "helt": "til dels",
    "bred": "smal",
    "villigt": "uvilligt",
    "villighed": "uvilje",
    "vind": "slappe af",
    "visdom": "dårskab",
    "klog": "tåbelig",
    "bekymre": "berolige",
    "værdiløs": "værdifuld",
    "værdig": "uværdig",
    "skriftlig": "uskrevet",
    "ung": "alderen",
    "zenit": "lavpunkt",
}



"""
Created on Fri Sep  3 12:29:39 2021

@author: alfonso
"""

from emoatlas.resources import _load_dictionary
import numpy as np
import random
import itertools
import warnings
import pandas as pd


def _emotion_distribution(
    wordlist,
    emotion_lexicon=None,
    normalize_strategy="none",
    emotions=[],
    language="english",
):
    """
    Count emotions in given wordlist.

    Required arguments:
    ----------
    *wordlist*:
        A list of words.

    *emotion_lexicon*:
        A lexicon with every word-emotion association. By default, the NRCLexicon will be loaded.

    *normalize_strategy*:
        A string, whether to normalize emotion scores over the number of words. Accepted values are:
            'none': no normalization at all
            'text_lenght': normalize emotion counts over the total text length
            'emotion_words': normalize emotion counts over the number of words associated to an emotion

    *emotions*:
        A list of emotions, depending on the model required. Default is Pluthick's wheel of emotions model.

    *language*:
        Language of the text. Full support is offered for the languages supported by Spacy:
            Catalan, Chinese, Danish, Dutch, English, French, German, Greek, Japanese, Italian, Lithuanian,
            Macedonian, Norvegian, Polish, Portuguese, Romanian, Russian, Spanish.
        Limited support for other languages is available.

    Returns:
    ----------
    *emo_distr*:
        A list of lists. Each entry is a list of emotions associated to a word in wordlist.
    """

    if not emotion_lexicon:
        emotion_lexicon = _load_dictionary(language)
        emotion_lexicon = (
            emotion_lexicon.groupby("word")["emotion"].apply(list).to_dict()
        )

    if not emotions:
        emotions = [
            "anger",
            "trust",
            "surprise",
            "disgust",
            "joy",
            "sadness",
            "fear",
            "anticipation",
        ]

    emo_distr = [
        emotion_lexicon[word] for word in wordlist if word in emotion_lexicon.keys()
    ]

    return emo_distr


def _emotion_words(
    wordlist,
    emotion_lexicon=None,
    normalize_strategy="none",
    emotions=[],
    language="english",
):
    """
    Count emotions in given wordlist. Returns the words most associated to emotions.

    Required arguments:
    ----------
    *wordlist*:
        A list of words.

    *emotion_lexicon*:
        A lexicon with every word-emotion association. By default, the NRCLexicon will be loaded.

    *normalize_strategy*:
        A string, whether to normalize emotion scores over the number of words. Accepted values are:
            'none': no normalization at all
            'text_lenght': normalize emotion counts over the total text length
            'emotion_words': normalize emotion counts over the number of words associated to an emotion

    *emotions*:
        A list of emotions, depending on the model required. Default is Pluthick's wheel of emotions model.

    *language*:
        Language of the text. Full support is offered for the languages supported by Spacy:
            Catalan, Chinese, Danish, Dutch, English, French, German, Greek, Japanese, Italian, Lithuanian,
            Macedonian, Norvegian, Polish, Portuguese, Romanian, Russian, Spanish.
        Limited support for other languages is available.

    Returns:
    ----------
    *emo_words:
        A dictionary. For each emotion, a tuple (word, n_occurrences). Max 10 words per emotion.
    """

    emotion_lexicon = _load_dictionary(language)

    if not emotions:
        emotions = [
            "anger",
            "trust",
            "surprise",
            "disgust",
            "joy",
            "sadness",
            "fear",
            "anticipation",
        ]

    worddf = pd.Series(wordlist).value_counts()
    worddf = pd.DataFrame({"word": worddf.index, "count": worddf.values})

    worddf = (
        pd.merge(worddf, emotion_lexicon, on="word")
        .sort_values("count", ascending=False)
        .reset_index()
    )

    emo_words = {
        emo: worddf.loc[worddf["emotion"] == emo,].head(10) for emo in emotions
    }

    return emo_words


def _samples(baseline_distr, sample_size, emotions=[], n_samples=300, epsilon=0.0):
    """
    Get mean and std of emotions distribution in randomized samples of words-related emotions taken from the baseline.

    Required arguments:
    ----------
    *baseline_distr*:
        Distribution of emotions in the baseline.

    *sample_size*:
        An integer. How big the random samples are supposed to be.

    *emotions*:
        List of emotions.

    *n_samples*:
        An integer, how many samples to take from the baseline.

    Returns:
    ----------

    *wordlist*:
        A list of words. Words preceded by negations have been eliminated.
    """

    emo_sample_tot = {emo: [] for emo in emotions}

    for k in range(n_samples):

        Lrandom = random.choices(baseline_distr, k=sample_size)

        # chain emotions
        randemo = list(itertools.chain(*Lrandom))
        randemo = {emo: randemo.count(emo) for emo in set(randemo)}

        # Update results
        emo_sample_tot = {
            k: emo_sample_tot.get(k) + [randemo.get(k, 0)] for k in set(emo_sample_tot)
        }

    # Get mean and std for each emotion
    emo_sample_tot = {
        k: {"mean": np.mean(val), "std": np.std(val)}
        for k, val in emo_sample_tot.items()
    }
    if any([emo_sample_tot[k]["std"] == 0 for k in emo_sample_tot.keys()]):

        if epsilon > 0:
            raise ValueError(
                "There are too few emotions in input text. When sampling a list of words from a baseline, "
                "some emotions don't appear at all. The resulting z-score would result in a division by zero, "
                "and it would be not meaningful. Please input a larger text. You can also provide epsilon > 0, "
                "avoiding the DivisionByZero error."
            )
        else:
            for k in emo_sample_tot.keys():
                if emo_sample_tot[k]["std"] == 0:
                    emo_sample_tot[k]["std"] = epsilon

            warnings.warn(
                "The emotion distribution of the samples of the baseline had some 0s. In order to avoid a division "
                "by zero, an epsilon = 0.1 has been forced as denominator when computing z-scores. "
                "Resampling is strongly suggested."
            )

    return emo_sample_tot
